package com.fjhx.victoriatourist.service.jd.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.victoriatourist.entity.jd.dto.JdOrderSelectDto;
import com.fjhx.victoriatourist.entity.jd.po.JdOrder;
import com.fjhx.victoriatourist.entity.jd.po.JdOrderDetails;
import com.fjhx.victoriatourist.entity.jd.vo.JdOrderDetailsVo;
import com.fjhx.victoriatourist.mapper.jd.JdOrderDetailsMapper;
import com.fjhx.victoriatourist.mapper.jd.JdOrderMapper;
import com.fjhx.victoriatourist.service.jd.JdOrderDetailsService;
import com.fjhx.victoriatourist.service.jd.JdOrderService;
import com.fjhx.victoriatourist.service.jd.constants.RunParamConstant;
import com.fjhx.wms.entity.stock.emums.JournalType;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockJournal;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.fjhx.wms.service.stock.StockJournalService;
import com.fjhx.wms.service.stock.StockService;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * <p>
 * 京东订单明细 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-04-20
 */
@Service
public class JdOrderDetailsServiceImpl extends ServiceImpl<JdOrderDetailsMapper, JdOrderDetails> implements JdOrderDetailsService {
    @Resource
    private StockService stockService;
    @Resource
    private StockJournalService stockJournalService;
    @Resource
    private StockJournalDetailsService stockJournalDetailsService;
    @Resource
    private ProductInfoService productInfoService;
    @Resource
    private JdOrderMapper jdOrderMapper;

    @Override
    public List<JdOrderDetailsVo> findListByJdOrderId(Long jdOrderId) {
        List<JdOrderDetails> list = lambdaQuery().eq(JdOrderDetails::getJdOrderId, jdOrderId).list();
        return BeanUtil.copyToList(list, JdOrderDetailsVo.class);
    }

    @Override
    public Page<JdOrderDetailsVo> jdOrderDetailExList(JdOrderSelectDto dto) {
        Page<JdOrderDetails> page = lambdaQuery()
                .eq(JdOrderDetails::getHasEx, "1")
                .like(ObjectUtil.isNotNull(dto.getOrderId()), JdOrderDetails::getOrderId, dto.getOrderId())
                .eq(StrUtil.isNotBlank(dto.getHandleStatus()), JdOrderDetails::getHandleStatus, dto.getHandleStatus())
                .eq(StrUtil.isNotBlank(dto.getExHandle()), JdOrderDetails::getExHandle, dto.getExHandle())
                .page(new Page<>(dto.getPageNum(), dto.getPageSize()));

        List<JdOrderDetails> records = page.getRecords();
        List<JdOrderDetailsVo> voList = BeanUtil.copyToList(records, JdOrderDetailsVo.class);
        Page<JdOrderDetailsVo> voPage = BeanUtil.copyProperties(page, Page.class);

        productInfoService.attributeAssign(voList, JdOrderDetails::getProductId, (item, product) -> {
            item.setProductCode(product.getCode());
            item.setProductName(product.getName());
            item.setProductCustomCode(product.getCustomCode());
        });

        voPage.setRecords(voList);

        return voPage;
    }


    @Override
    public void dealEx(JdOrderDetailsVo vo) {
        JdOrderDetails byId = getById(vo.getId());
        if (ObjectUtil.isNull(byId)){
            throw new RuntimeException("数据不存在");
        }
        byId.setHasEx(RunParamConstant.NO_EXCEPTION);
        byId.setHandleStatus(RunParamConstant.EXCEPTION_STATE_TREATED);
        byId.setExHandle(vo.getExHandle());
        byId.setHandleTime(new Date());
        Long userId = SecurityUtils.getUserId();
        byId.setHandleUser(userId);
        //对应处理异常详细方法
        switch (vo.getExHandle()){
            case RunParamConstant.MULTI_CARGO_REDUCE_STOCK:
                //弹窗提示，出库，减少库存
                reduceStock(vo);
                break;
            case RunParamConstant.SHIPPED_DAMAGE:
            case RunParamConstant.SHIPPED_CLAIM:
            case RunParamConstant.BARCODE_ERROR_DAMAGE:
                fillRemark(vo, byId);
                break;
            case RunParamConstant.SHIPPED_NOT_REDUCE_STOCK:
            case RunParamConstant.BARCODE_ERROR_NOT_RETURN:
                //仅弹窗提醒与修改状态，不做其他操作，修改状态在上面已经统一处理，预留位置。
                break;
            case RunParamConstant.SHORTAGE_RETURN:
                //选择仓库，增加库存
                addStock(vo);
                break;
            case RunParamConstant.EXCESS_RETURN:
            case RunParamConstant.BARCODE_ERROR_RETURN:
                fillExpressNoAndAddStock(vo, byId);
                break;
            case RunParamConstant.EXCESS_NOT_RETURN:
                //增减库存
                addAndReduceStock(vo);
                break;
            case RunParamConstant.EXCESS_DAMAGE:
                //填备注增减库存
                fillRemarkAndAddReduceStock(vo, byId);
                break;
            default:
                throw new RuntimeException("异常处理状态异常");
        }
        updateById(byId);

        //判断当前采购单下所有明细是不是都已经无异常，如果是，修改采购单异常状态为无异常
        List<JdOrderDetails> list = lambdaQuery().eq(JdOrderDetails::getJdOrderId, byId.getJdOrderId()).list();
        if(CollectionUtil.isEmpty(list)){
            throw new RuntimeException("采购单明细不存在");
        }
        boolean allMatch = list.stream().allMatch(jdOrderDetails -> StrUtil.equals(jdOrderDetails.getHasEx(), RunParamConstant.NO_EXCEPTION));
        if(allMatch){
            //修改采购单状态为已处理
            JdOrder jdOrder = jdOrderMapper.selectById(byId.getJdOrderId());
            jdOrder.setHasEx(RunParamConstant.NO_EXCEPTION);
            jdOrderMapper.updateById(jdOrder);
        }
    }


    /**
     * 填充备注
     * @author hj
     * @date 2024/1/15 22:41
     */
    private void fillRemark(JdOrderDetailsVo vo, JdOrderDetails jdOrderDetails){
        String remark = vo.getRemark();
        if(StrUtil.isBlank(remark)){
            throw new RuntimeException("备注不能为空");
        }
        jdOrderDetails.setRemark(remark);
    }

    /**
     * 填写快递单号并增加库存
     * @author hj
     * @date 2024/1/15 22:42
     */
    private void fillExpressNoAndAddStock(JdOrderDetailsVo vo, JdOrderDetails jdOrderDetails) {
        String expressNo = vo.getExpressNo();
        if (StrUtil.isBlank(expressNo)) {
            throw new RuntimeException("快递单号不能为空");
        }
        jdOrderDetails.setExpressNo(expressNo);
        addStock(vo);
    }

    /**
     * 增减库存
     * @author hj
     * @date 2024/1/15 22:42
     */
    private void addAndReduceStock(JdOrderDetailsVo vo) {
        addStock(vo);
        reduceStock(vo);
    }


    /**
     * 填备注增减库存
     * @author hj
     * @date 2024/1/15 22:42
     */
    private void fillRemarkAndAddReduceStock(JdOrderDetailsVo vo, JdOrderDetails jdOrderDetails) {
        fillRemark(vo, jdOrderDetails);
        addStock(vo);
        reduceStock(vo);
    }

    /**
     * 增加库存
     * @param vo
     */
    private void addStock(JdOrderDetailsVo vo) {
        //操作库存及记录
        operateStock(vo.getAddWarehouseId(), RunParamConstant.INBOUND, vo.getAddQuantity());
    }

    /**
     * 减少库存
     * @author hj
     * @date 2024/1/15 22:44
     */
    private void reduceStock(JdOrderDetailsVo vo) {
        //操作库存及记录
        operateStock(vo.getReduceWarehouseId(), RunParamConstant.OUTBOUND, vo.getReduceQuantity());
    }



    /**
     * 操作库存及记录
     * @param warehouseId 仓库id
     * @param type 类型 1入库 2出库 3维多利亚冻结库存入库 4维多利亚待出库次品库存
     * @param quantity 操作数量
     */
    private void operateStock(Long warehouseId, int type, BigDecimal quantity) {
        if (ObjectUtil.isNull(warehouseId) || quantity.compareTo(BigDecimal.ZERO) <= 0) {
            throw new RuntimeException("参数错误");
        }

        //要操作的数据
        List<Stock> stockList = new ArrayList<>();
        Stock stock = new Stock();
        stock.setProductId(1L);
        stock.setQuantity(quantity);
        stockList.add(stock);

        //操作库存
        stockService.ModifyInventory(type, stockList, warehouseId);

        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setOpType(1);//1入库2出库
        stockJournal.setType(JournalType.JD_BACK_IN.getDetailType());//流水类型 JournalType枚举
        stockJournal.setCode("流水编号");//流水编号，没规范赋值业务编号也行
        stockJournal.setWarehouseId(warehouseId);//仓库id
        stockJournal.setBusinessId(1L);//业务id
        stockJournal.setLinkCode("业务编号");//业务编号
        stockJournalService.save(stockJournal);

        //保存出入库流水明细
        List<StockJournalDetails> stockJournalDetailsList = new ArrayList<>();
        StockJournalDetails stockJournalDetails = new StockJournalDetails();
        stockJournalDetails.setId(IdWorker.getId());
        stockJournalDetails.setBusinessDetailsId(1L);//业务明细id
        stockJournalDetails.setQuantity(BigDecimal.ZERO);//操作数量;
        stockJournalDetails.setProductId(1L);//操作产品id
        stockJournalDetailsList.add(stockJournalDetails);
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

}
